@@ -23,10 +23,11 @@
 #include <Plasma/Applet>
 
 #include "ui_fileWatcherConfig.h"
+#include "ui_filtersConfig.h"
 
 class QFile;
-class QFileSystemWatcher;
 class QTextDocument;
+class KDirWatch;
 class FileWatcherTextItem;
 
 
@@ -49,6 +50,7 @@ class FileWatcher : public Plasma::Applet
   private slots:
     void newData();
     void loadFile(const QString& path);
+    void fileDeleted(const QString& path);
 
     void createConfigurationInterface(KConfigDialog *parent);
 
@@ -56,14 +58,19 @@ class FileWatcher : public Plasma::Applet
     void updateRows();
 
     QFile *file;
-    QFileSystemWatcher *watcher;
+    KDirWatch *watcher;
     FileWatcherTextItem *textItem;
     QTextStream *textStream;
     QTextDocument *textDocument;
 
-    QString m_tmpPath;
+    QString m_currentPath;
+
+    QStringList m_filters;
+    bool m_showOnlyMatches;
+    bool m_useRegularExpressions;
 
     Ui::fileWatcherConfig ui;
+    Ui::filtersConfig filtersUi;
 };
 
 K_EXPORT_PLASMA_APPLET(fileWatcher, FileWatcher)
