@@ -40,7 +40,8 @@
 #include <KServiceTypeTrader>
 #include <kglobalsettings.h>
 
-#include <plasma/paintutils.h>
+#include <Plasma/PaintUtils>
+#include <Plasma/DataEngine>
 
 #include <math.h>
 
@@ -56,9 +57,9 @@ Frame::Frame(QObject *parent, const QVariantList &args)
     setHasConfigurationInterface(true);
     setAcceptDrops(true);
     setAcceptsHoverEvents(true);
-    setCacheMode(QGraphicsItem::NoCache);
+    setCacheMode(QGraphicsItem::DeviceCoordinateCache);
     resize(400, 300);
-    m_mySlideShow = new SlideShow();
+    m_mySlideShow = new SlideShow(this);
     if (args.count()) {
         m_currentUrl = args.value(0).toString();
     } else {
@@ -68,27 +69,6 @@ Frame::Frame(QObject *parent, const QVariantList &args)
 
 Frame::~Frame()
 {
-    delete m_mySlideShow;
-}
-
-void Frame::dataUpdated(const QString &name, const Plasma::DataEngine::Data &data)
-{
-    Q_UNUSED(name);
-    QDate mCurrentDate = QDate::currentDate();
-    const QString identifier = m_potdProvider + ':' + mCurrentDate.toString(Qt::ISODate);
-
-    QImage _picture = data[identifier].value<QImage>();
-
-    if (!_picture.isNull()) {
-        m_picture = _picture;
-        m_pixmapCache = QPixmap();
-	QSizeF sizeHint = contentSizeHint();
-	if (geometry().size() != sizeHint) {
-            resize(sizeHint); 
-	} else {        
-            update();
-	}
-    }
 }
 
 void Frame::init()
@@ -120,9 +100,7 @@ void Frame::init()
     m_swOutline = 8;
 
     // Initialize the slideshow timer
-    m_slideShowTimer = new QTimer(this);
-    connect(m_slideShowTimer, SIGNAL(timeout()), this, SLOT(updatePicture()));
-    m_slideShowTimer->setInterval(m_slideshowTime * 1000);
+    connect(m_mySlideShow, SIGNAL(pictureUpdated()), this, SLOT(updatePicture()));
 
     initSlideShow();
     if (frameReceivedUrlArgs) {
@@ -131,14 +109,13 @@ void Frame::init()
     }
 
     m_menuPresent = false;
-    updateMenu();
 }
 
 void Frame::updateMenu()
 {
-    if (!m_menuPresent && !m_potd && m_currentUrl.path() != "Default") {
-        kDebug() << "Current path: " << m_currentUrl.path();
-        m_openPicture = new QAction(SmallIcon("image-x-generic"), i18n("&Open Picture..."), this);
+    if (hasAuthorization("LaunchApp") && ! (m_menuPresent || m_potd || (m_currentUrl.path() == "Default" && m_mySlideShow->currentUrl() == "Default"))) {
+        kDebug() << "Current url: " << m_currentUrl.url();
+        m_openPicture = new QAction(SmallIcon("image-x-generic"), i18n("&Open Picture..."), this);       
         m_actions.append(m_openPicture);
         connect(m_openPicture, SIGNAL(triggered(bool)), this , SLOT(slotOpenPicture()));
         m_menuPresent = true;
@@ -159,10 +136,13 @@ QList<QAction*> Frame::contextualActions()
 
 void Frame::slotOpenPicture()
 {
+    if (!hasAuthorization("LaunchApp")) {
+        return;
+    }
     KUrl url;
 
     if (m_slideShow) {
-        url = m_mySlideShow->getCurrentUrl();
+        url = m_mySlideShow->currentUrl();
     } else {
         url = m_currentUrl;
     }
@@ -174,25 +154,26 @@ void Frame::slotOpenPicture()
 
 void Frame::constraintsEvent(Plasma::Constraints constraints)
 {
-    
     if (constraints & Plasma::FormFactorConstraint) {
         setBackgroundHints(Plasma::Applet::NoBackground);
-	m_pixmapCache = QPixmap();
-	if (formFactor() == Plasma::Horizontal) {
-	     m_frameOutline = 0;
-             m_swOutline = 4;
-	} else if (formFactor() == Plasma::Vertical) {
-	     m_frameOutline = 0;
-             m_swOutline = 4;
+        if (formFactor() == Plasma::Horizontal) {
+            m_frameOutline = 0;
+            m_swOutline = 4;
+        } else if (formFactor() == Plasma::Vertical) {
+            m_frameOutline = 0;
+            m_swOutline = 4;
         } else {
-	     m_frameOutline = 8;
-             m_swOutline = 8;
-	     //Restore widget geometry to image proportions
-	     resize(contentSizeHint());
-	}
-    }  
-    
-    if (constraints & Plasma::SizeConstraint) {   
+            m_frameOutline = 8;
+            m_swOutline = 8;
+            //Restore widget geometry to image proportions
+            resize(contentSizeHint());
+            emit appletTransformedItself();
+        }
+
+        updatePicture();
+    }
+
+    if (constraints & Plasma::SizeConstraint) {
         //If on panel, keep geometry to 4:3 ratio
         if(formFactor() == Plasma::Vertical) {
             setMinimumSize(QSizeF(0, boundingRect().width()/1.33));
@@ -200,7 +181,7 @@ void Frame::constraintsEvent(Plasma::Constraints constraints)
         } else if(formFactor() == Plasma::Horizontal) {
             setMinimumSize(QSizeF(boundingRect().height()*1.33,0));
             setMaximumSize(QSizeF(boundingRect().height()*1.33,-1));
-        } else { 
+        } else {
             setMinimumSize(QSizeF());
             setMaximumSize(QSizeF());
         }
@@ -209,11 +190,11 @@ void Frame::constraintsEvent(Plasma::Constraints constraints)
 
 QSizeF Frame::contentSizeHint() const
 {
-    if (!m_picture.isNull() && (formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter)){
-        QSizeF sizeHint = m_picture.size();
-        qreal maxSize = geometry().width() > geometry().height() ? geometry().width() : geometry().height();
-        sizeHint.scale(maxSize, maxSize, Qt::KeepAspectRatio);
-        return sizeHint;
+    if (!m_pictureSize.isEmpty() && (formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter)){
+        const qreal maxSize = geometry().width() > geometry().height() ? geometry().width() : geometry().height();
+        QSize size = m_pictureSize;
+        size.scale(maxSize, maxSize, Qt::KeepAspectRatio);
+        return size;
     } else {
         return geometry().size();
     }
@@ -221,27 +202,24 @@ QSizeF Frame::contentSizeHint() const
 
 void Frame::updatePicture()
 {
-    QImage newImage = m_mySlideShow->getImage();
-
-    if (!newImage.isNull()) {
-        m_picture = newImage;
-        m_pixmapCache = QPixmap();
-	QSizeF sizeHint = contentSizeHint();
-	if (geometry().size() != sizeHint) {
-            resize(sizeHint); 
-	} else {        
-            update();
-	}
+    QImage picture = m_mySlideShow->image();
+    m_pictureSize = picture.size();
+    QSizeF sizeHint = contentSizeHint();
+    if (geometry().size() != sizeHint) {
+        resize(sizeHint);
+        emit appletTransformedItself();
     }
+    updateMenu();
+    update();
 }
 
 void Frame::addDir()
 {
     KDirSelectDialog dialog(KUrl(), true);
     if (dialog.exec()) {
-        QString path = dialog.url().path();
+        QString path = dialog.url().url();
         if (!m_slideShowPaths.contains(path)) {
-            m_configDialog->ui.slideShowDirList->addItem(path);
+            m_configDialog->imageUi.slideShowDirList->addItem(path);
         }
         updateButtons();
     }
@@ -249,17 +227,17 @@ void Frame::addDir()
 
 void Frame::removeDir()
 {
-    int row = m_configDialog->ui.slideShowDirList->currentRow();
+    int row = m_configDialog->imageUi.slideShowDirList->currentRow();
     if (row != -1) {
-        m_configDialog->ui.slideShowDirList->takeItem(row);
+        m_configDialog->imageUi.slideShowDirList->takeItem(row);
         updateButtons();
     }
 }
 
 void Frame::updateButtons()
 {
-    int row = m_configDialog->ui.slideShowDirList->currentRow();
-    m_configDialog->ui.removeDirButton->setEnabled(row != -1);
+    int row = m_configDialog->imageUi.slideShowDirList->currentRow();
+    m_configDialog->imageUi.removeDirButton->setEnabled(row != -1);
 }
 
 void Frame::createConfigurationInterface(KConfigDialog *parent)
@@ -270,25 +248,29 @@ void Frame::createConfigurationInterface(KConfigDialog *parent)
     foreach(const KService::Ptr &service, services) {
         const QString service_name(service->name());
         const QVariant service_identifier(service->property("X-KDE-PlasmaPoTDProvider-Identifier", QVariant::String).toString());
-        m_configDialog->ui.potdComboBox->insertItem(m_configDialog->ui.potdComboBox->count(), service_name, service_identifier);
+        m_configDialog->imageUi.potdComboBox->insertItem(m_configDialog->imageUi.potdComboBox->count(), service_name, service_identifier);
     }
 
-    QStandardItemModel* model = static_cast<QStandardItemModel*>(m_configDialog->ui.pictureComboBox->model());
+    QStandardItemModel* model = static_cast<QStandardItemModel*>(m_configDialog->imageUi.pictureComboBox->model());
     QStandardItem* item = model->item(2);
-    if (services.isEmpty())
-        item->setFlags(item->flags() & ~Qt::ItemIsEnabled);
-    else
-        item->setFlags(item->flags() | Qt::ItemIsEnabled);
 
-    parent->addPage(m_configDialog, i18n("General"), icon());
+    if (item) {
+        if (services.isEmpty())
+            item->setFlags(item->flags() & ~Qt::ItemIsEnabled);
+        else
+            item->setFlags(item->flags() | Qt::ItemIsEnabled);
+    }
+
+    parent->addPage(m_configDialog->imageSettings, i18n("Image"), icon());
+    parent->addPage(m_configDialog->appearanceSettings, i18n("Appearance"), icon());
     parent->setDefaultButton(KDialog::Ok);
     parent->showButtonSeparator(true);
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
 
-    connect(m_configDialog->ui.removeDirButton, SIGNAL(clicked()), this, SLOT(removeDir()));
-    connect(m_configDialog->ui.addDirButton, SIGNAL(clicked()), this, SLOT(addDir()));
-    connect(m_configDialog->ui.slideShowDirList, SIGNAL(currentRowChanged(int)), this, SLOT(updateButtons()));
+    connect(m_configDialog->imageUi.removeDirButton, SIGNAL(clicked()), this, SLOT(removeDir()));
+    connect(m_configDialog->imageUi.addDirButton, SIGNAL(clicked()), this, SLOT(addDir()));
+    connect(m_configDialog->imageUi.slideShowDirList, SIGNAL(currentRowChanged(int)), this, SLOT(updateButtons()));
 
     m_configDialog->setRoundCorners(m_roundCorners);
     m_configDialog->setSmoothScaling(m_smoothScaling);
@@ -297,26 +279,24 @@ void Frame::createConfigurationInterface(KConfigDialog *parent)
     m_configDialog->setFrameColor(m_frameColor);
 
     if (m_slideShow) {
-        m_configDialog->ui.pictureComboBox->setCurrentIndex(1);
+        m_configDialog->imageUi.pictureComboBox->setCurrentIndex(1);
     } else if (m_potd) {
-        m_configDialog->ui.pictureComboBox->setCurrentIndex(2);
+        m_configDialog->imageUi.pictureComboBox->setCurrentIndex(2);
     } else {
-        m_configDialog->ui.pictureComboBox->setCurrentIndex(0);
+        m_configDialog->imageUi.pictureComboBox->setCurrentIndex(0);
     }
 
-    m_configDialog->ui.randomCheckBox->setCheckState(m_random ? Qt::Checked : Qt::Unchecked);
-    m_configDialog->ui.recursiveCheckBox->setCheckState(m_recursiveSlideShow ? Qt::Checked : Qt::Unchecked);
+    m_configDialog->imageUi.randomCheckBox->setCheckState(m_random ? Qt::Checked : Qt::Unchecked);
+    m_configDialog->imageUi.recursiveCheckBox->setCheckState(m_recursiveSlideShow ? Qt::Checked : Qt::Unchecked);
 
-    m_configDialog->ui.potdComboBox->setCurrentIndex(m_configDialog->ui.potdComboBox->findData(m_potdProvider));
+    m_configDialog->imageUi.potdComboBox->setCurrentIndex(m_configDialog->imageUi.potdComboBox->findData(m_potdProvider));
 
     m_configDialog->setCurrentUrl(m_currentUrl);
-    m_configDialog->ui.slideShowDirList->clear();
-    m_configDialog->ui.slideShowDirList->addItems(m_slideShowPaths);
-    m_configDialog->ui.removeDirButton->setEnabled(!m_slideShowPaths.isEmpty());
-    m_configDialog->ui.slideShowDelay->setTime(QTime(m_slideshowTime / 3600, (m_slideshowTime / 60) % 60, m_slideshowTime % 60));
-    m_configDialog->previewPicture(m_picture);
-    m_configDialog->show();
-    m_configDialog->raise();
+    m_configDialog->imageUi.slideShowDirList->clear();
+    m_configDialog->imageUi.slideShowDirList->addItems(m_slideShowPaths);
+    m_configDialog->imageUi.removeDirButton->setEnabled(!m_slideShowPaths.isEmpty());
+    m_configDialog->imageUi.slideShowDelay->setTime(QTime(m_slideshowTime / 3600, (m_slideshowTime / 60) % 60, m_slideshowTime % 60));
+    m_configDialog->previewPicture(m_mySlideShow->image());
 }
 
 void Frame::configAccepted()
@@ -334,10 +314,12 @@ void Frame::configAccepted()
     m_frameColor = m_configDialog->frameColor();
     cg.writeEntry("frameColor", m_frameColor);
 
-    if (m_configDialog->ui.pictureComboBox->currentIndex() == 1) {
+    bool wasPotd = m_potd;
+
+    if (m_configDialog->imageUi.pictureComboBox->currentIndex() == 1) {
         m_slideShow = true;
         m_potd = false;
-    }  else if (m_configDialog->ui.pictureComboBox->currentIndex() == 2)   {
+    }  else if (m_configDialog->imageUi.pictureComboBox->currentIndex() == 2)   {
         m_slideShow = false;
         m_potd = true;
     }  else {
@@ -350,59 +332,67 @@ void Frame::configAccepted()
     m_currentUrl = m_configDialog->currentUrl();
     cg.writeEntry("url", m_currentUrl);
     cg.writeEntry("slideshow", m_slideShow);
-    m_recursiveSlideShow = m_configDialog->ui.recursiveCheckBox->checkState() == Qt::Checked ? true : false;
+    m_recursiveSlideShow = m_configDialog->imageUi.recursiveCheckBox->checkState() == Qt::Checked ? true : false;
     cg.writeEntry("recursive slideshow", m_recursiveSlideShow);
     m_slideShowPaths.clear();
     QStringList dirs;
-    for (int i = 0; i < m_configDialog->ui.slideShowDirList->count(); i++) {
-        m_slideShowPaths << m_configDialog->ui.slideShowDirList->item(i)->text();
+    for (int i = 0; i < m_configDialog->imageUi.slideShowDirList->count(); i++) {
+        m_slideShowPaths << m_configDialog->imageUi.slideShowDirList->item(i)->text();
     }
     cg.writeEntry("slideshow paths", m_slideShowPaths);
 
-    QTime timerTime = m_configDialog->ui.slideShowDelay->time();
+    QTime timerTime = m_configDialog->imageUi.slideShowDelay->time();
     m_slideshowTime = timerTime.second() + timerTime.minute() * 60 + timerTime.hour() * 3600;
-    m_slideShowTimer->setInterval(m_slideshowTime * 1000);
     cg.writeEntry("slideshow time", m_slideshowTime);
 
-    m_potdProvider = m_configDialog->ui.potdComboBox->itemData(m_configDialog->ui.potdComboBox->currentIndex()).toString();
+    QString potdProvider = m_configDialog->imageUi.potdComboBox->itemData(m_configDialog->imageUi.potdComboBox->currentIndex()).toString();
+
+    if ((wasPotd && !m_potd) || (m_potd && potdProvider != m_potdProvider)) {
+        // if we go from potd to no potd, or if the provider changes, then we first want to
+        // stop the potd engine
+        stopPotd();
+    }
+
+    m_potdProvider = potdProvider;
     cg.writeEntry("potdProvider", m_potdProvider);
     cg.writeEntry("potd", m_potd);
 
+    initSlideShow();
+    
     // Creates the menu if the settings have changed from "Default" to sth. else
     updateMenu();
 
-    initSlideShow();
-
     emit configNeedsSaving();
 }
 
+void Frame::stopPotd()
+{
+    Plasma::DataEngine *engine = dataEngine("potd");
+    QDate mCurrentDate = QDate::currentDate();
+    const QString identifier = m_potdProvider + ':' + mCurrentDate.toString(Qt::ISODate);
+    engine->disconnectSource(identifier, m_mySlideShow);
+    m_potd = false;
+}
+
 void Frame::initSlideShow()
 {
+    m_mySlideShow->setUpdateInterval(0);
+
     if (m_slideShow) {
         m_mySlideShow->setDirs(m_slideShowPaths, m_recursiveSlideShow);
         m_mySlideShow->setRandom(m_random);
-        m_slideShowTimer->start();
+        m_mySlideShow->setUpdateInterval(m_slideshowTime * 1000);
     } else if (m_potd) {
-	if(m_slideShowTimer->isActive()) {
-	    m_slideShowTimer->stop();
-	}
-        
-	Plasma::DataEngine *engine = dataEngine("potd");
-        if (!engine) {
-            kDebug() << " Engine potd can't be created";
-            return;
-        }
+        Plasma::DataEngine *engine = dataEngine("potd");
         QDate mCurrentDate = QDate::currentDate();
         const QString identifier = m_potdProvider + ':' + mCurrentDate.toString(Qt::ISODate);
 
-        engine->disconnectSource(identifier, this);
-        engine->connectSource(identifier, this);
+        engine->connectSource(identifier, m_mySlideShow);
 
-        const Plasma::DataEngine::Data data = engine->query(identifier);
+//FIXME: why is there a manual kicking of the engine?        const Plasma::DataEngine::Data data = engine->query(identifier);
     } else { //no slideshow so no random stuff
         m_mySlideShow->setRandom(false);
-        m_mySlideShow->setImage(m_currentUrl.path());
-        m_slideShowTimer->stop();
+        m_mySlideShow->setImage(m_currentUrl.url());
     }
 
     if (!m_potd) {
@@ -423,24 +413,27 @@ void Frame::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
 void Frame::dropEvent(QGraphicsSceneDragDropEvent *event)
 {
     KUrl droppedUrl = (KUrl::List::fromMimeData(event->mimeData())).at(0);
+    kDebug() << "dropped URL" << droppedUrl.url();
     if (droppedUrl.protocol() == "desktop") {
-	KUrl tmpUrl = KGlobalSettings::desktopPath() + droppedUrl.path();
-	droppedUrl = tmpUrl;
-	}
+        KUrl tmpUrl = KGlobalSettings::desktopPath() + droppedUrl.path();
+        droppedUrl = tmpUrl;
+    }
     // If the url is a local directory start slideshowmode
     if (droppedUrl.isLocalFile() && QFileInfo(droppedUrl.path()).isDir()) {
         m_slideShowPaths.clear();
-	m_slideShowPaths.append(droppedUrl.path());
+        m_slideShowPaths.append(droppedUrl.path());
         if (!m_slideShow) {
             m_slideShow = true;
         }
     } else {
+        kDebug() << "Remote URL" << droppedUrl.url();
         m_currentUrl = droppedUrl;
         if (m_slideShow) {
             m_slideShow = false;
         }
     }
-    m_potd = false;
+
+    stopPotd();
     initSlideShow();
 
     KConfigGroup cg = config();
@@ -450,39 +443,37 @@ void Frame::dropEvent(QGraphicsSceneDragDropEvent *event)
     emit configNeedsSaving();
 }
 
-void Frame::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option,
-                           const QRect &rect)
+void Frame::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &rect)
 {
-    if (m_pixmapCache.isNull() ||
-            geometry().toRect().size() != m_pixmapCache.size()) {
-	    //kDebug() << "Paint cache!!!";
-        paintCache(option, geometry().toRect().size());
+    Q_UNUSED(option)
+    if (m_slideShow) {
+        // temporarily suspend the slideshow to allow time for loading the image
+        m_mySlideShow->setUpdateInterval(0);
     }
 
-    p->drawPixmap(rect, m_pixmapCache, rect);
-}
-
-void Frame::paintCache(const QStyleOptionGraphicsItem *option,
-                       const QSize &contentsSize)
-{
-    Q_UNUSED(option);
-    m_pixmapCache = QPixmap(contentsSize);
-    m_pixmapCache.fill(Qt::transparent);
+    QImage picture = m_mySlideShow->image();
+    const QSize contentsSize = rect.size();
 
+    if (picture.isNull()) {
+        if (m_slideShow) {
+            // unsuspend the slideshow to allow time for loading the image
+            m_mySlideShow->setUpdateInterval(m_slideshowTime * 1000);
+        }
+        return;
+    }
 
     int roundingFactor = 12 * m_roundCorners;
     int swRoundness = roundingFactor + m_frameOutline / 2 * m_frame * m_roundCorners;
 
-    QRect frameRect = m_pixmapCache.rect().adjusted(m_swOutline, m_swOutline,
-                      -m_swOutline, -m_swOutline); //Pretty useless.
+    QRect frameRect(QPoint(0, 0), contentsSize);
+    frameRect.adjust(m_swOutline, m_swOutline, -m_swOutline, -m_swOutline); //Pretty useless.
 
     Qt::TransformationMode transformationMode = m_smoothScaling ? Qt::SmoothTransformation : Qt::FastTransformation;
     //TODO check if correct
-    QImage scaledImage = m_picture.scaled(frameRect.size(), Qt::KeepAspectRatio, transformationMode);
+    QImage scaledImage = picture.scaled(frameRect.size(), Qt::KeepAspectRatio, transformationMode);
     frameRect = QRect(QPoint(frameRect.x() + (frameRect.width() - scaledImage.width()) / 2,
                              frameRect.y() + (frameRect.height() - scaledImage.height()) / 2), scaledImage.size());
 
-    QPainter *p = new QPainter(&m_pixmapCache);
     QRect shadowRect;
     if (m_frame) {
         shadowRect = frameRect.adjusted(-m_frameOutline + 1, -m_frameOutline + 1,
@@ -542,7 +533,10 @@ void Frame::paintCache(const QStyleOptionGraphicsItem *option,
         p->drawPath(framePath);
     }
 
-    delete p;
+    if (m_slideShow) {
+        // unsuspend the slideshow to allow time for loading the image
+        m_mySlideShow->setUpdateInterval(m_slideshowTime * 1000);
+    }
 }
 
 #include "frame.moc"
