@@ -18,8 +18,8 @@
  */
 
 #include "LancelotApplet.h"
-#include "KDebug"
-#include "KIcon"
+#include <KIcon>
+#include <climits>
 
 #include <QDBusInterface>
 #include <QDBusReply>
@@ -31,6 +31,11 @@
 #include "lancelot_interface.h"
 #include "../LancelotConfig.h"
 
+#define SIZE_CMIN 16
+#define SIZE_CPREF 32
+#define SIZE_CMAX 64
+#define SPACING 8
+
 class LancelotApplet::Private {
 public:
     Private(LancelotApplet * parent)
@@ -39,6 +44,8 @@ public:
         lancelot(NULL), offline(false)
     {
         q->setLayout(layout);
+        layout->setContentsMargins(0, 0, 0, 0);
+        layout->setSpacing(0);
         lancelot = new org::kde::lancelot::App(
             "org.kde.lancelot", "/Lancelot",
             QDBusConnection::sessionBus()
@@ -87,51 +94,36 @@ public:
             // Error connecting to Lancelot via d-bus
             // setFailedToLaunch(true);
             return;
-        } else {
-            // Creating buttons...
-            for (int i = 0; i < replyIDs.value().size(); i++) {
-                Lancelot::HoverIcon * button = new Lancelot::HoverIcon(
-                    KIcon(replyIcons.value().at(i)), "", q);
+        }
 
-                connect(
-                    button, SIGNAL(activated()),
-                    & signalMapper, SLOT(map())
-                );
-
-                signalMapper.setMapping(button, replyIDs.value().at(i));
-
-                connect(
-                    button, SIGNAL(clicked()),
-                    & signalMapperToggle, SLOT(map())
-                );
-
-                signalMapperToggle.setMapping(button, replyIDs.value().at(i));
-
-                layout->addItem(button);
-                button->setActivationMethod(clickActivation?(Lancelot::ClickActivate):(Lancelot::HoverActivate));
-                buttons << button;
+        // Creating buttons...
+        for (int i = 0; i < replyIDs.value().size(); i++) {
+            if (categsHide.contains(replyIDs.value().at(i))) {
+                continue;
             }
+
+            Lancelot::HoverIcon * button = new Lancelot::HoverIcon(
+                KIcon(replyIcons.value().at(i)), "", q);
+
+            connect(
+                button, SIGNAL(activated()),
+                & signalMapper, SLOT(map())
+            );
+
+            signalMapper.setMapping(button, replyIDs.value().at(i));
+
+            connect(
+                button, SIGNAL(clicked()),
+                & signalMapperToggle, SLOT(map())
+            );
+
+            signalMapperToggle.setMapping(button, replyIDs.value().at(i));
+
+            layout->addItem(button);
+            button->setActivationMethod(clickActivation?(Lancelot::ClickActivate):(Lancelot::HoverActivate));
+
+            buttons << button;
         }
-
-    }
-
-    void resize()
-    {
-        QSizeF size = q->size();
-
-        if (q->formFactor() == Plasma::Vertical) {
-            layout->setPreferredSize(size.width(), size.width() * (showCategories?4:1));
-            // q->resize(size.width(), size.width() * buttons.count());
-        } else if (q->formFactor() == Plasma::Horizontal) {
-            layout->setPreferredSize(size.height() * (showCategories?4:1), size.height());
-            // q->resize(size.height() * buttons.count(), size.height());
-        } else {
-            size = size.expandedTo(QSizeF(IconSize(KIconLoader::Desktop), IconSize(KIconLoader::Desktop)));
-            size = QSizeF(size.height() * buttons.count(), size.height());
-            layout->setPreferredSize(size);
-            q->resize(size);
-        }
-        layout->updateGeometry();
     }
 
     void toggleHide()
@@ -148,6 +140,7 @@ public:
     bool showCategories;
     QString mainIcon;
     bool clickActivation;
+    QStringList categsHide;
 
     QSignalMapper signalMapper;
     QSignalMapper signalMapperToggle;
@@ -182,10 +175,16 @@ LancelotApplet::LancelotApplet(QObject * parent,
             parent));
     connect(action, SIGNAL(triggered(bool)), d->lancelot, SLOT(showMenuEditor()));
 
-    d->waitClick.setInterval(1000); // 1 sec
+    d->waitClick.setInterval(500); // 1/2 sec
     d->waitClick.setSingleShot(true);
 }
 
+// void LancelotApplet::paint(QPainter * p,
+//     const QStyleOptionGraphicsItem * options, QWidget * widget)
+// {
+//     p->fillRect(options->rect, QBrush(QColor()));
+// }
+
 LancelotApplet::~LancelotApplet()
 {
     delete d;
@@ -197,6 +196,7 @@ void LancelotApplet::loadConfig()
     d->showCategories =  (kcg.readEntry("show", "main") != "main");
     d->mainIcon = kcg.readEntry("icon", "lancelot");
     d->clickActivation = (kcg.readEntry("activate", "click") == "click");
+    d->categsHide = kcg.readEntry("hiddenCategories", QStringList());
 }
 
 void LancelotApplet::saveConfig()
@@ -205,6 +205,7 @@ void LancelotApplet::saveConfig()
     kcg.writeEntry("show", (d->showCategories?"categories":"main"));
     kcg.writeEntry("icon", d->mainIcon);
     kcg.writeEntry("activate", (d->clickActivation?"click":"hover"));
+    kcg.writeEntry("hiddenCategories", d->categsHide);
     save(kcg);
 
     m_configMenu.saveConfig();
@@ -213,14 +214,13 @@ void LancelotApplet::saveConfig()
 void LancelotApplet::applyConfig()
 {
     d->layout->setContentsMargins(0, 0, 0, 0);
-    d->layout->setSpacing(0);
+    d->layout->setSpacing(SPACING);
     if (d->showCategories) {
         d->createCategoriesButtons();
     } else {
         d->createMainButton();
     }
     emit configNeedsSaving();
-    d->resize();
     update();
     setAspectRatioMode(Plasma::KeepAspectRatio);
 }
@@ -239,13 +239,16 @@ void LancelotApplet::init()
         & d->signalMapperToggle, SIGNAL(mapped(const QString &)),
         this, SLOT(toggleLancelotSection(const QString &))
     );
+    KGlobal::locale()->insertCatalog("lancelot");
 }
 
 void LancelotApplet::showLancelot()
 {
     if (d->offline) return;
 
-    d->waitClick.start();
+    if (!d->lancelot->isShowing()) {
+        d->waitClick.start();
+    }
     QPoint position = popupPosition(QSize());
     d->lancelot->show(position.x(), position.y());
 }
@@ -271,7 +274,9 @@ void LancelotApplet::showLancelotSection(const QString & section)
 {
     if (d->offline) return;
 
-    d->waitClick.start();
+    if (!d->lancelot->isShowing()) {
+        d->waitClick.start();
+    }
     QPoint position = popupPosition(QSize());
     d->lancelot->showItem(position.x(), position.y(), section);
 }
@@ -291,6 +296,7 @@ void LancelotApplet::configAccepted()
     d->showCategories = m_config.showCategories();
     d->mainIcon = m_config.icon();
     d->clickActivation = m_config.clickActivation();
+    d->categsHide = m_config.showingCategories(false);
     applyConfig();
     saveConfig();
     d->lancelot->configurationChanged();
@@ -304,6 +310,7 @@ void LancelotApplet::createConfigurationInterface(KConfigDialog * parent)
     m_config.setShowCategories(d->showCategories);
     m_config.setIcon(d->mainIcon);
     m_config.setClickActivation(d->clickActivation);
+    m_config.setShowingCategories(d->categsHide, false);
     parent->addPage(appletConfig, i18n("Applet"),
             "application-x-plasma", i18n("Lancelot Launcher Applet"));
 
@@ -327,12 +334,17 @@ void LancelotApplet::constraintsEvent(Plasma::Constraints constraints)
             d->layout->setOrientation(Qt::Horizontal);
         }
     }
-    d->resize();
+
+    if (formFactor() == Plasma::Horizontal) {
+        setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding));
+    } else if (formFactor() == Plasma::Vertical) {
+        setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
+    }
 }
 
 void LancelotApplet::hoverLeaveEvent(QGraphicsSceneHoverEvent * event)
 {
-    Q_UNUSED(event);
+    Plasma::Applet::hoverLeaveEvent(event);
     d->offline = false;
 }
 
@@ -345,5 +357,19 @@ QList< QAction * > LancelotApplet::contextualActions()
     return result;
 }
 
+QSizeF LancelotApplet::sizeHint(Qt::SizeHint which, const QSizeF & constraint) const
+{
+    QSizeF hint = Plasma::Applet::sizeHint(which, constraint);
+    if (formFactor() == Plasma::Horizontal &&
+            (which == Qt::MaximumSize || size().height() <= KIconLoader::SizeLarge)) {
+        hint.setWidth(size().height() * d->buttons.size());
+    } else if (formFactor() == Plasma::Vertical &&
+            (which == Qt::MaximumSize || size().width() <= KIconLoader::SizeLarge)) {
+        hint.setHeight(size().width() * d->buttons.size());
+    }
+
+    return hint;
+}
+
 #include "LancelotApplet.moc"
 
