@@ -19,14 +19,16 @@
 
 #include "Runner.h"
 #include <KRun>
-#include <KDebug>
 #include <KIcon>
 #include <KLocalizedString>
 #include <KStandardDirs>
 #include "FavoriteApplications.h"
+#include "logger/Logger.h"
 
 #include <plasma/abstractrunner.h>
 
+#define SLEEP 600
+
 namespace Models {
 
 Runner::Runner(QString search)
@@ -44,15 +46,35 @@ Runner::~Runner()
 {
 }
 
-QString Runner::searchString()
+QString Runner::searchString() const
 {
     return m_searchString;
 }
 
+QString Runner::runnerName() const
+{
+    return m_runnerName;
+}
+
+void Runner::setRunnerName(const QString & name)
+{
+    m_runnerName = name;
+}
+
 void Runner::setSearchString(const QString & search)
 {
     m_searchString = search.trimmed();
+    m_timer.start(SLEEP, this);
+}
 
+void Runner::timerEvent(QTimerEvent * event)
+{
+    BaseModel::timerEvent(event);
+    if (event->timerId() != m_timer.timerId()) {
+        return;
+    }
+
+    m_timer.stop();
     if (m_searchString.isEmpty()) {
         clear();
         add(
@@ -63,7 +85,11 @@ void Runner::setSearchString(const QString & search)
         );
         valid = false;
     } else {
-        m_runnerManager->launchQuery(search);
+        if (m_runnerName.isEmpty()) {
+            m_runnerManager->launchQuery(m_searchString);
+        } else {
+            m_runnerManager->launchQuery(m_searchString, m_runnerName);
+        }
     }
 }
 
@@ -87,12 +113,10 @@ void Runner::setQueryMatches(const QList< Plasma::QueryMatch > & m)
 
         while (newMatchIt.hasNext()) {
             Plasma::QueryMatch match = newMatchIt.next();
-            kDebug() << match.id() << match.runner()->id() << match.runner()->objectName();
             QStringList data;
             data << match.id();
             data << match.runner()->id();
             data << match.data().toString();
-            kDebug() << data;
 
             add(
                 match.text(),
@@ -114,7 +138,9 @@ void Runner::load()
 void Runner::activate(int index)
 {
     if (!valid) return;
-    m_runnerManager->run(m_items[index].data.value< QStringList >().at(0));
+    QString data = itemAt(index).data.value< QStringList >().at(0);
+    Logger::instance()->log("run-model", data);
+    m_runnerManager->run(data);
     m_runnerManager->reset();
     changeLancelotSearchString(QString());
     hideLancelotWindow();
@@ -123,18 +149,39 @@ void Runner::activate(int index)
 bool Runner::hasContextActions(int index) const
 {
     if (!valid) return false;
-    kDebug() << m_items[index].data.value< QString >();
-    return (m_items[index].data.value< QStringList >().at(1) == "Application");
+    if (itemAt(index).data.value< QStringList >().at(1) == "services") {
+        return true;
+    }
+
+    QString id = itemAt(index).data.value< QStringList >().at(0);
+    foreach (Plasma::QueryMatch match, m_runnerManager->matches()) {
+        if (match.id() == id) {
+            if (m_runnerManager->actionsForMatch(match).size() > 0) {
+                return true;
+            }
+        }
+    }
+
+    return false;
 }
 
-void Runner::setContextActions(int index, QMenu * menu)
+void Runner::setContextActions(int index, Lancelot::PopupMenu * menu)
 {
     if (!valid) return;
 
-    if (m_items[index].data.value< QStringList >().at(1) == "Application") {
+    if (itemAt(index).data.value< QStringList >().at(1) == "services") {
         menu->addAction(KIcon("list-add"), i18n("Add to Favorites"))
             ->setData(QVariant(0));
     }
+
+    QString id = itemAt(index).data.value< QStringList >().at(0);
+    foreach (Plasma::QueryMatch match, m_runnerManager->matches()) {
+        if (match.id() == id) {
+            foreach (QAction * action, m_runnerManager->actionsForMatch(match)) {
+                menu->addAction(action->icon(), action->text());
+            }
+        }
+    }
 }
 
 void Runner::contextActivate(int index, QAction * context)
@@ -143,12 +190,33 @@ void Runner::contextActivate(int index, QAction * context)
 
     if (context->data().toInt() == 0) {
         KService::Ptr service = KService::serviceByStorageId(
-                m_items[index].data.value< QStringList >().at(2));
+                itemAt(index).data.value< QStringList >().at(2));
         if (service) {
-;           FavoriteApplications::instance()
+            FavoriteApplications::instance()
                 ->addFavorite(service->entryPath());
         }
     }
 }
 
+QMimeData * Runner::mimeData(int index) const
+{
+    if (!valid) return NULL;
+
+    if (itemAt(index).data.value< QStringList >().at(1) == "services") {
+        KService::Ptr service = KService::serviceByStorageId(
+                itemAt(index).data.value< QStringList >().at(2));
+        return BaseModel::mimeForService(service);
+    }
+
+    return NULL;
+}
+
+void Runner::setDropActions(int index,
+            Qt::DropActions & actions, Qt::DropAction & defaultAction)
+{
+    Q_UNUSED(index);
+    actions = Qt::CopyAction;
+    defaultAction = Qt::CopyAction;
+}
+
 } // namespace Models
