@@ -17,24 +17,23 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#ifndef LANCELOTAPP_MODELS_SYSTEMSERVICES_H_
-#define LANCELOTAPP_MODELS_SYSTEMSERVICES_H_
+#ifndef LANCELOTAPP_MODELS_SYSTEMSERVICES_H
+#define LANCELOTAPP_MODELS_SYSTEMSERVICES_H
 
-#include "BaseModel.h"
+#include "FolderModel.h"
 
 namespace Models {
 
-class SystemServices : public BaseModel {
+class SystemServices : public FolderModel {
     Q_OBJECT
 public:
     SystemServices();
     virtual ~SystemServices();
 
-protected:
-    void load();
+    static QString path();
 
 };
 
 }
 
-#endif /* LANCELOTAPP_MODELS_SYSTEMSERVICES_H_ */
+#endif /* LANCELOTAPP_MODELS_SYSTEMSERVICES_H */
