@@ -25,24 +25,22 @@
 
 #include <QPainter>
 
+#include <KConfigDialog>
+#include <KDebug>
+
 #include <Plasma/Svg>
 #include <Plasma/Theme>
-#include <KDebug>
-#include <KConfigDialog>
+#include <Plasma/ToolTipManager>
 
 #include "phases.cpp"
 
-
-
 Luna::Luna(QObject *parent, const QVariantList &args)
-    : Plasma::Applet(parent, args)
+    : Plasma::Applet(parent, args),
+      counter(-1)
 {
     setHasConfigurationInterface(true);
     setAspectRatioMode(Plasma::ConstrainedSquare);
-    size = 82;
-    resize(QSize(size, size));
-
-    counter = -1;
+    resize(QSize(82, 82));
 }
 
 void Luna::init()
@@ -56,9 +54,9 @@ void Luna::init()
         return;
     }
 
-    KConfigGroup cg = config();
+    Plasma::ToolTipManager::self()->registerWidget(this);
 
-    northHemisphere = cg.readEntry("northHemisphere", true);
+    northHemisphere = config().readEntry("northHemisphere", true);
 
     connectToEngine();
 }
@@ -68,17 +66,6 @@ Luna::~Luna()
     delete m_theme;
 }
 
-void Luna::constraintsEvent(Plasma::Constraints constraints)
-{
-    if (constraints & Plasma::SizeConstraint) {
-        if (formFactor() == Plasma::Horizontal) {
-            setMinimumWidth(geometry().height());
-        } else if (formFactor() == Plasma::Vertical) {
-            setMinimumHeight((int)geometry().width());
-        }
-    }
-}
-
 void Luna::connectToEngine()
 {
     Plasma::DataEngine* timeEngine = dataEngine("time");
@@ -97,10 +84,9 @@ void Luna::createConfigurationInterface(KConfigDialog *parent)
 {
     QWidget *widget = new QWidget();
     ui.setupUi(widget);
-    parent->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
-    parent->addPage(widget, parent->windowTitle(), icon());
+    parent->addPage(widget, i18n("General"), icon());
 
     ui.northenRadio->setChecked(northHemisphere);
     ui.southernRadio->setChecked(!northHemisphere);
@@ -108,19 +94,16 @@ void Luna::createConfigurationInterface(KConfigDialog *parent)
 
 void Luna::configAccepted()
 {
-    KConfigGroup cg = config();
-
     northHemisphere = ui.northenRadio->isChecked();
 
-    cg.writeEntry("northHemisphere", northHemisphere);
+    config().writeEntry("northHemisphere", northHemisphere);
 
     update();
 
     emit configNeedsSaving();
 }
 
-void Luna::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option,
-const QRect &contentsRect)
+void Luna::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
 {
     Q_UNUSED(option)
 
@@ -138,9 +121,7 @@ const QRect &contentsRect)
 
 void Luna::calcStatus(time_t time)
 {
-    #if 0
-    Plasma::ToolTipData::ToolTipData toolTipData;
-    #endif
+    Plasma::ToolTipContent toolTipData;
 
     uint lun = 0;
     time_t last_new = 0;
@@ -187,9 +168,7 @@ void Luna::calcStatus(time_t time)
 
     if ( fm.daysTo( now ) == 0 ) {
         counter = 14;
-        #if 0
-        toolTipData.mainText = i18n( "Full Moon" );
-        #endif
+        toolTipData.setMainText( i18n( "Full Moon" ) );
         return;
     } else if ( counter <= 15 && counter >= 13 ) {
         counter = 14 + fm.daysTo( now );
@@ -232,53 +211,48 @@ void Luna::calcStatus(time_t time)
 
     switch (counter) {
     case 0:
-        #if 0
-        toolTipData.mainText = i18n("New Moon");
-        #endif
+        toolTipData.setMainText( i18n("New Moon") );
         break;
+
     case 1:
     case 2:
     case 3:
     case 4:
     case 5:
     case 6:
-        #if 0
-        toolTipData.mainText = i18np("Waxing Crescent (New Moon was yesterday)", "Waxing Crescent (%1 days since New Moon)", counter );
-        #endif
+        toolTipData.setMainText( i18np("Waxing Crescent (New Moon was yesterday)", "Waxing Crescent (%1 days since New Moon)", counter ) );
         break;
+
     case 7:
-        #if 0
-        toolTipData.mainText = i18n("First Quarter");
-        #endif
+        toolTipData.setMainText( i18n("First Quarter") );
         break;
+
     case 8:
     case 9:
     case 10:
     case 11:
     case 12:
     case 13:
-        #if 0
-        toolTipData.mainText = i18np( "Waxing Gibbous (Tomorrow is Full Moon)", "Waxing Gibbous (%1 days to Full Moon)", -fm.daysTo( now ) );
-        #endif
+        toolTipData.setMainText( i18np( "Waxing Gibbous (Tomorrow is Full Moon)", "Waxing Gibbous (%1 days to Full Moon)", -fm.daysTo( now ) ) );
         break;
+
     case 14:
         assert( false );
         break;
+
     case 15:
     case 16:
     case 17:
     case 18:
     case 19:
     case 20:
-        #if 0
-        toolTipData.mainText = i18np("Waning Gibbous (Yesterday was Full Moon)", "Waning Gibbous (%1 days since Full Moon)", fm.daysTo( now ) );
-        #endif
+        toolTipData.setMainText( i18np("Waning Gibbous (Yesterday was Full Moon)", "Waning Gibbous (%1 days since Full Moon)", fm.daysTo( now ) ) );
         break;
+
     case 21:
-        #if 0
-        toolTipData.mainText = i18n("Last Quarter");
-        #endif
+        toolTipData.setMainText( i18n("Last Quarter") );
         break;
+
     case 22:
     case 23:
     case 24:
@@ -287,16 +261,14 @@ void Luna::calcStatus(time_t time)
     case 27:
     case 28:
         kDebug() << "nn.days " << ln.daysTo( now ) << " " << nn.daysTo( now );
-        #if 0
-        toolTipData.mainText = i18np("Waning Crescent (Tomorrow is New Moon)", "Waning Crescent (%1 days to New Moon)", -nn.daysTo( now ) );
-        #endif
+        toolTipData.setMainText( i18np("Waning Crescent (Tomorrow is New Moon)", "Waning Crescent (%1 days to New Moon)", -nn.daysTo( now ) ) );
         break;
+
     default:
         kFatal() << "coolo can't count\n";
     }
-    #if 0
-    setToolTip(toolTipData);
-    #endif
+
+    Plasma::ToolTipManager::self()->setContent(this, toolTipData);
     update();
 }
 
